@@ -265,7 +265,7 @@ MainWindow::setupActions()
 
    (new KWidgetAction( m_positionSlider, i18n("Position Slider"), 0, 0, 0, ac, "position_slider" ))->setAutoSized( true );
 
-   new VolumeAction( toolBar(), ac );
+   m_volumeAction = new VolumeAction( toolBar(), ac );
 }
 
 void
